// Fichier Barillet.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                Code source de la librairie           *
 *					de la carte barillet                *
 *                                                      *
 * .Date de cration : 06/02/07                         *
 * .Auteur : Xavier Maquignon                           *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la carte barillet     *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/

#include <p18f452.h>		// Librairie du PIC utilis
#include <delays.h>			// Delais pour le check-up


#include "I2C_Master.h"		// Librairie pour la liaison srie
#include "Barillet.h"		// Entte de la librairie de gestion du robot

// --------------------------------------------------------------------------


// --------------------------------------------------------------------------
/* Place le servo de dpose pour faire un dpose vers la gauche
 * Arguments : aucun
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int Servo_Gauche_Barillet(void)
{
	return (I2C_Send( ADR_I2C_BARILLET, BARI_SERVO_GAUCHE ));	
}	



// --------------------------------------------------------------------------
/* Place le servo de dpose pour faire un dpose vers la droite
 * Arguments : aucun
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int Servo_Droite_Barillet(void)
{
	return (I2C_Send( ADR_I2C_BARILLET, BARI_SERVO_DROITE ));	
}


// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
/* Vidange intgralement le barillet dans l'ordre le plus rapide
 * Arguments : aucun
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int Vidange_Barillet( void )
{
	return (I2C_Send( ADR_I2C_BARILLET, BARI_VIDANGE ));
}




// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
/* Envoie l'ordre de stopper le barillet
 * Arguments : aucun
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int Stop_Barillet( void )
{
	return (I2C_Send( ADR_I2C_BARILLET, BARI_STOP ));
}



// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
/* Informe le barillet que l'on joue la couleur bleu
 * Arguments : aucun
 * Note : par dafaut, le barillet joue la couleur rouge
 *
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int Jouer_Bleu_Barillet(void)
{
	return (I2C_Send( ADR_I2C_BARILLET, JOUER_BLEU ));	
}	



// --------------------------------------------------------------------------
/* ralise une dpose de notre barillet de notre ct
 * Arguments : aucun
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int DeposeBalle_Chez_Nous_Barillet( void ){
	return I2C_Send( ADR_I2C_BARILLET, BARI_DEPOT_CHEZ_NOUS);
}


// --------------------------------------------------------------------------
/* ralise une dpose de notre barillet chez l'adversaire
 * Arguments : aucun
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int DeposeBalle_Adverse_Barillet( void ){
	return I2C_Send( ADR_I2C_BARILLET, BARI_DEPOT_CHEZ_ADV);
}


// --------------------------------------------------------------------------
/* Dpose une balle de la couleur passe en paramtre
 * Arguments : 
 *  1 ou BLEUE	 : dpose une balle bleue
 *  2 ou BLANCHE : dpose une balle blanche
 *  3 ou ROUGE	 : dpose une balle rouge
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int DeposeBalle_Barillet( char couleur ){
	switch( couleur ){
		case BLEUE :
			return I2C_Send( ADR_I2C_BARILLET, BARI_DEPOT_BLEUE );
			break;

		case BLANCHE :
			return I2C_Send( ADR_I2C_BARILLET, BARI_DEPOT_BLANCHE  );
			break;

		case ROUGE :
			return I2C_Send( ADR_I2C_BARILLET, BARI_DEPOT_ROUGE  );
			break;
		
		default :
			return -3;
	}
}

// --------------------------------------------------------------------------
/* Tire une balle de la couleur passe en paramtre
 * Arguments : 
 *  1 ou BLEUE	 : tire une balle bleue
 *  2 ou BLANCHE : tire une balle blanche
 *  3 ou ROUGE	 : tire une balle rouge
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int TirBalle_Barillet( char couleur ){
	switch( couleur ){
		case BLEUE :
			return I2C_Send( ADR_I2C_BARILLET, BARI_TIR_BLEUE );
			break;

		case BLANCHE :
			return I2C_Send( ADR_I2C_BARILLET, BARI_TIR_BLANCHE  );
			break;

		case ROUGE :
			return I2C_Send( ADR_I2C_BARILLET, BARI_TIR_ROUGE  );
			break;
		
		default :
			return -3;
	}
}



// --------------------------------------------------------------------------
/* Lit l'tat et le contenu du barillet
 * Arguments : pointeur vers une struture de type TEtatBarillet
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int Etat_Barillet( TEtatBarillet* etat ){
	unsigned char Data[2];										// Donnes brutes
	int Ret;													// Valeur de retour de l'envoie sur l'I2C

	Ret = I2C_Receive_x(ADR_I2C_BARILLET,2,Data);				// Lecture des donnes
	
	etat->slot[0] = (Data[0] & 0xC0)>>6;						// Mise en forme du slot 0
	etat->slot[1] = (Data[0] & 0x30)>>4;
	etat->slot[2] = (Data[0] & 0x0C)>>2;
	etat->slot[3] = (Data[0] & 0x03);
	etat->slot[4] = (Data[1] & 0xC0)>>6;

	etat->avale = (Data[1] & 0x20)!=0;							// Mise en forme de l'tat du processeur
	etat->busy = (Data[1] & 0x10)!=0;							// Mise en forme de l'tat du processeur
	
	return Ret;													// Retourne la lecture de l'IC

}

// --------------------------------------------------------------------------
/* Fonction qui indique si le barillet est occup ou non
 * Arguments : aucun
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass, il n'est pas occup
 *   1 : le barillet est occup
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int Barillet_Busy(void)
{
	unsigned char Data[2];										// Donnes brutes
	int Ret;													// Valeur de retour de l'envoie sur l'I2C

	if (I2C_Receive_x(ADR_I2C_BARILLET,2,Data))					// Lecture des donnes
		return -1;												// En cas d'erreur sur l'I2C, on retourne -1
	TEMPO_INTER_REQUETES_I2C									// Tempo pour viter les lecture successives trop rapides
	return ((Data[1] & 0x10)!=0);								// On retourne l'tat du barillet
		
}	


// --------------------------------------------------------------------------
/* Fonction qui indique si le barillet est en train d'avaler une balle
 * Arguments : aucun
 * 
 * Paramtre retourn : 
 *   0 : tout s'est bien pass, il n'y a pas de balle
 *   1 : une balle est en cours de traitement
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
int Barillet_Avale(void)
{
	unsigned char Data[2];										// Donnes brutes
	int Ret;													// Valeur de retour de l'envoie sur l'I2C

	if (I2C_Receive_x(ADR_I2C_BARILLET,2,Data))					// Lecture des donnes
		return -1;												// En cas d'erreur sur l'I2C, on retourne -1
	TEMPO_INTER_REQUETES_I2C									// Tempo pour viter les lecture successives trop rapides
	return ((Data[1] & 0x20)!=0);								// On retourne le bit concern
		
}	

// --------------------------------------------------------------------------
/* Effectue un chek-up du barillet
 * Vidange le barillet   								(erreurs 1)
 * Lit le contenu et l'tat du barillet				 	(erreurs 2)
 * Vrifie que le barillet est vide						(erreurs 3)
 * Si ncessaire, change la couleur joue				(erreurs 4)
 * Pas de problme : on retourne 0 
*/
char Check_Up_Barillet(unsigned char NotreCouleur)
{
	TEtatBarillet etat;
	if (Vidange_Barillet()) return 1;							// Vidange le barillet
	Delay10KTCYx(100);											// Tempo pour le laisser dmarer
	while (Barillet_Busy());									// Attends que le barillet ne soit plus occup
	if (Etat_Barillet( &etat ))		return 2;					// Lit le contenu du barillet	
	if (etat.slot[0]!=0) return 3;								// Vrifie que le slot 0 est vide
	if (etat.slot[1]!=0) return 3;								// Vrifie que le slot 1 est vide
	if (etat.slot[2]!=0) return 3;								// Vrifie que le slot 2 est vide
	if (etat.slot[3]!=0) return 3;								// Vrifie que le slot 3 est vide
	if (etat.slot[4]!=0) return 3;								// Vrifie que le slot 4 est vide
	if (NotreCouleur==BLEUE)									// Si on joue bleu
		if (Jouer_Bleu_Barillet())								// On envoie l'information au barillet
			return (4);											// En cas de problme, on retourne 4
	return 0;	
}




// --------------------------------------------------------------------------
/* Renvoie le nombre de balles d'une couleurs dans le barillet
 * 
 * Pas de problme : on retourne le nombre de balles
 * -1 problme sur l'I2C
*/
char NbBalles_Barillet(unsigned char Couleur)
{
	TEtatBarillet etat;
	char i,Nb=0;

	if (Etat_Barillet( &etat ))		return -1;					// Lit le contenu du barillet
	TEMPO_INTER_REQUETES_I2C									// Tempo pour viter les lecture successives trop rapides
	
	if (Couleur==TOUTES)										// On souhaite connatre le nombre de balles toutes couleurs confondues
	{
		for (i=0;i<5;i++)										// Parcourt le tableau
			if (etat.slot[i]!=NONE) Nb++;						// Compte les balles
		return Nb;												// Retourne le nombre de balles
	}
	
	for (i=0;i<5;i++)											// On compte les balles de la couleur demande
		if (etat.slot[i]==Couleur) Nb++;						
	
	return Nb;													// Retourne le nombre de balles
}