// Fichier Robot.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                Code source de la librairie           *
 *					de la carte principale 				*
 *                                                      *
 * .Date de cration : 29/12/07                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la carte principale   *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/

#include <p18f452.h>		// Librairie du PIC utilis
#include <delays.h>			// Delais pour le check-up


#include "I2C_Master.h"		// Librairie pour la liaison srie
#include "Canon.h"		// Entte de la librairie de gestion du robot



// --------------------------------------------------------------------------
/* Commande le canon
 * Speed : moteur canon (vitesse entre 0 et 255)
 * 0 le canon est arrt, 
 * 255 le canon va  fond
 * Paramtre retourn : 
 *   0 : Tout s'est bien pass
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2 : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Commande_Canon(unsigned char Speed)
{
	return (I2C_Send(ADR_I2C_CANON,Speed));				// Retourne le rsultat de l'envoie sur l'I2C
}		



// --------------------------------------------------------------------------
/* Mesure la vitesse du canon
 * La fonction mesure la vitesse du canon
 *  entre -128 et +127 si tout s'est bien pass
 * 	0x0FFF en cas de problme dans la transmission
*/
int Mesure_Canon(void) 
{
	int Speed=I2C_Receive(ADR_I2C_CANON);				// Lit la vitesse via l'IC
	if (Speed==-1) 										// En cas de problme de transmission ..
		return (0x0FFF);								// .. on retourne 0x0FFF
	else												// si tout va bien
		return ((unsigned char)Speed);							// on retourne la vitesse caste en char
}


// --------------------------------------------------------------------------
/* Mesure la vitesse moyenne du canon sur N mesures
 * La fonction mesure la vitesse du canon
 *  entre -128 et +127 si tout s'est bien pass
 * 	0x0FFF en cas de problme dans la transmission
*/
int Mesure_Moyenne_Canon(int N)
{
	int i=0;
	long unsigned int Speed=0;
		
	for (i=0;i<N;i++)
	{
		Speed+=I2C_Receive(ADR_I2C_CANON);				// Lit la vitesse via l'IC
		Delay10KTCYx(1);
	}
		
	return (Speed/N);									// on retourne la vitesse caste en char
}



// --------------------------------------------------------------------------
/* Effectue un chek-up du canon
 * Lance le canon en avant						   	 (erreur 1)
 * Vrifie la vitesse du canon						 (erreur 2)  
 * Arrte le moteur									 (erreur 3)
 * Vrifie qu'il est arret							 (erreur 4)
 * Pas de problme : on retourne 0 
*/
unsigned char Check_Up_Canon(void)
{
	int i=0;
	if (Commande_Canon(CHECK_UP_VITESSE_CANON)) 	return 1; 		// Fait tourner le canon
	while (Mesure_Canon()<CHECK_UP_MESURE_CANON)					// Attends que la commande soit applique
	{
		Delay10KTCYx(CHECK_UP_DELAY);								// Attends un peu dans la boucle
		if (i++==20) return 2;										// Si le canon n'a pas atteind la vitesse, il y a un problme
	}
	if (Commande_Canon(0)) 							return 3;		// Arrte le moteur
	i=0;
	while (Mesure_Canon()!=0)										// Attends que le canon soit arrt
	{
		Delay10KTCYx(CHECK_UP_DELAY);								// Attends un peu dans la boucle
		if (i++==20) return 4;										// Si le canon n'a pas stopp, il y a un problme
	}
	return 0;														// Tout s'est bien pass
}	