
// Fichier Depart_Bleue.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                source de la librairie             	*
 *					Depart Bleue						*
 *                                                      *
 * .Date de cration : 23/04/08                         *
 * .Auteur : Jrmie LUMINEAU							*       
 *                                                      *
 * .Descriptions :                                      *
 *      - code du dpart bleue							*
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/

#include <delays.h>
#include "mystdio.h"


#include "strategie_bleu.h"
#include "Depart_bleu.h"
#include "lcd.h"
#include "StartUp.h"
#include "JoyStick.h"
#include "Propulsion.h"
#include "rouleaux.h"
#include "IO.h"
#include "I2C_Master.h"
#include "barillet.h"
#include "Horloge.h"
#include "led.h"
#include "Telemetres.h"
#include "serial.h"
#include "canon.h"
#include "Tourelle.h"
#include "Constante.h"
#include "Distributeur.h"
#include "Recal.h"



// --------------------------------------------------------------------------
/* Fonction qui fait la premire phase du match 
 * Ramasse les deux premires balles bleues sur la table
 * puis prend ce qu'il peut sur la table, et enfin, va aux distributeur si
 * ncessaire avant d'aller dposer dans le conteneur standard
*/

void Depart_Bleu( void ){
	unsigned char	etat=0;										// Etat courant de la machine principale
	unsigned char   Etat_Rouleaux=0;							// Etat courant de la machine  tat pour les rouleaux

	
	char 			i,config=0;									// Position de la balle bleu mobile
	int				MesureTelemetre;							// Distance mesure au tlmtre
	int 			correction=0;								// Correction de la position pour la prise des balles
	int vit=100;

	char ismoving=0;
	char transition=0;
	char adv_present=0;
	char nb_balles=0;
	
	StartHorloge();												// Dmarre le chrono sur 90 secondes



// ------------------------------------------------------------------
// Phase 1 : prendre, si possible, les 2 balles rouges et le maximum de balles blanches prsentes sur la table
//

	Angle_Propulsion (PI+0.010,15);								// S'oriente vers la 1ere balle rouge
	while (IsMove_Propulsion());
	
	
	config = Vote_Telemetre( BLEUE, 80, 120 );					// dtermine si une balle bleue est prsente
	/* test config*/
	print_int(config);
	print_str("\n");
	print_int(Moyenne_Telemetre_Bleu(1000));
	print_str("\n");
	//Joy_Click();
	
	Commande_Rouleaux(60, 60);									// Met les rouleaux en marche
	
	
	etat = 0;	
	
	while( etat!=255 ){
		if( transition )
			print_strint( ">depart=", etat, 1 );
			
		/* pour les tests 
		while( !Joy_Sw() ){
			int S, X, Y, A;
			
			Info_Propulsion( &S, &X, &Y, &A );
			
			print_strint( "X=", X, 0 );
			print_strint( ",Y=", Y, 0 );
			print_strint( ",A=", A, 1 );
				
		}
		Delay10KTCYx(50);
		 fin des tests */
		
		//Joy_Click();
		
		// mise  jour des nouvelles conditions de transition :
		ismoving = IsMove_Propulsion();
		
		nb_balles = NbBalles_Barillet(TOUTES);
		
		
		
		if( nb_balles==5 && etat!=150 && etat!=151){					// Si le barillet est plein
			Stop_Propulsion();											// arrter la propulsion
			etat = 255;													// passer dans l'tat 150
		}



		// --------------------------------------------------------------
		// Machine  tat pour les rouleaux
		
		switch (Etat_Rouleaux){
			case 0 :  
				if (NbBalles_Barillet(TOUTES)==5) 				// Si on a 5 balles dans le robot
					Etat_Rouleaux=1;							// On change d'tat 
					break;
			case 1 : 
					Commande_Rouleaux(-60, -60);				// On inverse le sens des rouleaux
					Etat_Rouleaux=2;							// On quitte cet tat pour ne pas rexecuter la commande 
					break;
		}
		
		
		

			
		adv_present = Adversaire_Devant()>=2;
				
		if( transition )
		{
			Stop_Propulsion();
			while (Etat_Propulsion()!='a');
		}

	
		Delay100TCYx(1);										// Attente entre deux cycles
	
		switch( etat ){
			
			case 0:
				transition = 1;
				if( config==PROCHE )									// si la 1re balle rouge est prsente
					etat = 1;	
				else if( config==LOIN )
					etat = 10;
				
				//Joy_Click();
				
				break;

// -------------------------------	config 1
				
			case 1:	
				if( transition ){
					GotoXY_Propulsion(2804,200,50);						// avancer un peu pour pouvoir tourner	
					transition=0;
				}else if( !ismoving ){
					transition = 1;
					etat = 2;	
				}
				break;

			case 2:	
				if( transition ){
					Angle_Propulsion (PI-0.65,30);						// s'orienter vers la blanche fixe
					transition=0;
				}else if( !ismoving ){
					transition = 1;
					etat = 3;	
				}
				break;
							
			case 3:	
				if( transition ){
//					GotoXY_Propulsion(2240, 660, 100);					// Va chercher la blanche fixe
					GotoXY_Propulsion(2270, 710, 90);					// Va chercher la blanche fixe
					transition=0;
				}else if( !ismoving ){
					transition = 1;
					etat = 4;	
				}
				break;
							
			case 4:	
				if( transition ){
					Angle_Propulsion (-PI/2.0-0.1,30);						// S'oriente vers la 2me bleue
					transition=0;
				}else if( !ismoving ){
					transition = 1;
					etat = 5;	
				}
				break;
							
			case 5:	
				if( transition ){
					GotoXY_Propulsion(2200,350, 100);					// Va la chercher 
					transition=0;
				}else if( !ismoving ){
					transition = 1;
					etat = 40;	
				}
				break;
// -------------------------------	dbut config 2 & 3					
				
							
			case 10:	
				if( transition ){
					Angle_Propulsion (PI-0.294,20);						// S'oriente vers la suite de la trajectoire (aller chercher les balles blanches)
					transition=0;
				}else if( !ismoving  ){
					transition = 1;
				//	correction=-50;										// correction ncessaire pour assurer la suite de la trajectoire
					etat = 11;	
				}
				break;

			case 11 :
				if( transition ){
//					GotoXY_Propulsion(805, 370, 100);					// Va la chercher
					GotoXY_Propulsion(2200, 400, 100);					// Va la chercher
					transition=0;
				}else if( !ismoving  ){
					transition = 1;
					etat = 12;	
				}
				break;

			case 12 :
				if( transition ){
					Angle_Propulsion (PI,20);							// S'oriente pour regarder si la 2me bleue est prsente
					transition=0;
				}else if( !ismoving  ){
					transition = 1;
					
					config = Vote_Telemetre( BLEUE, 100, 150 );			// dtermine  si la 2me rouge est prsente
					
					/* test config*/
					print_int(config);
					print_str("\n");
					print_int(Moyenne_Telemetre_Bleu(1000));
					print_str("\n");
					
					//Joy_Click() ;
					
					if ( config==PROCHE )								// si la balle rouge est l
					{	//correction=0;									// on va aller la chercher
						etat = 20;
					}
					else if( config==LOIN )								// si la balle rouge n'est pas l
					{	//correction=-80;	
						etat = 30;
					}
				}
				break;

// -------------------------------	suite config 3						

			case 20 :
				if( transition ){
					GotoXY_Propulsion(1848, 400, 100);					// on va la chercher
					transition=0;
				}else if( !ismoving  ){
					transition = 1;
					etat = 40;	
				}
				break;

// -------------------------------	suite config 2						

			case 30 :
				if( transition ){
					Angle_Propulsion (PI+0.365,15);						// S'oriente vers la 2me balle bleue
					transition=0;
				}else if( !ismoving  ){
					transition = 1;
					etat = 31;	
				}
				break;

			case 31 :
			if( transition ){
//					GotoXY_Propulsion(1800, 225, 100);					// choper la 2me bleue
					GotoXY_Propulsion(1790, 245, 100);					// choper la 2me bleue
					transition=0;
				}else if( !ismoving  ){
					transition = 1;
					etat = 40;	
				}
				break;
// -------------------------------	fin config 2&3					

					
			case 40:
				if( transition ){
					GotoXY_Propulsion(2100, 800, -60);			// recule au distri blanc
					transition=0;
				}else if( !ismoving  ){
					transition = 1;
					etat = 41;	
				}
				break;

			
			case 41 :
				if( transition ){
					Angle_Propulsion (-0.15,30);						// S'oriente vers le distri blanc
					transition=0;
				}else if( !ismoving  ){
					transition = 1;
					etat = 42;	
				}
				break;

			case 42:
				if( transition ){
					GotoXY_Propulsion(2600,750, 80);		// aller chercher la blanche
					transition=0;
				}else if( !ismoving  ){
					transition = 1;
					etat = 43;	
				}
				break;
			
			case 43:

				if( transition ){
					Angle_Propulsion (0,30);						// S'oriente vers le distributeur blanc
					transition=0;
				}else if( !ismoving  ){

					transition = 1;
					etat = 44;	
				}
				break;
			
			case 44:
				Distributeur_Blanc_Bleu(3,1);						// prendre des balles blanches
				transition = 1;
				etat = 45;	
				break;

			case 45:
				if( transition ){
					GotoXY_Propulsion(2300,500,-45);					// reculer en face du distributeur rouge
					transition=0;
				}else if( !ismoving  ){
					transition = 1;
					etat = 46;											
				}
				break;
				
			case 46:
				if( transition ){
					Angle_Propulsion(-PI/2.0,30);					// s'orienter face au distri bleu
					transition=0;
				}else if( !ismoving  ){
					transition = 1;
					etat = 47;										
				}
				break;
					
			case 47:
				Distributeur_Bleu(2,1);							// faire le complment en rouges si ncessaire
				transition = 1;
				etat = 255;											
				break;
				
			default: 
				etat = 255;
				break;

		} // fin switch(etat)
	
	} // fin while(etat!=255)
	
	
	print_str( "fin" );
}