// Fichier I2C_Master.h
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                Librairie I2c							*
 *                                                      *
 * .Date de cration : 29/12/07                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - Entte des fonction de l'I2C					*
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/
#ifndef __I2C_MASTER_H_
#define __I2C_MASTER_H_


#define   	DELAY_ENVOI_MULTIPLE    		Delay10TCYx(32);	// Dlai entre deux envoies en continu

#define 	TRIS_SDA						TRISCbits.TRISC4	// Ligne de donnes
#define 	TRIS_SCL						TRISCbits.TRISC3	// Horloge


// --------------------------------------------------------------------------
/* Initialise les broches du bus I2C en sortie
 * Configure la liaison I2c  en maitre, 7bits, 100Kbits/s
 * Pas de paramtre
*/
void I2C_Init(void);


// --------------------------------------------------------------------------
/* Lit une donnes sur le bus I2C
 * Arguments :
 *  - Le premier paramtre est l'adresse de l'esclave
 * 
 * Paramtre retourn : 
 *   Valeur lue
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/

int I2C_Receive(unsigned char Addr);



// --------------------------------------------------------------------------
/* Lit des donnes sur le bus I2C
 * Arguments :
 *  - Le premier paramtre est l'adresse de l'esclave
 *  - Le second paramtre est le nombre de donnes  lire [0:255]
 *  - Le dernier paramtre est un tableau contenant les donnes  mmoriser
 * 
 * Parametre renvoy : 
 * 	- 1 : La transmission s'est bien passe
 *  - 0 : pas d'acknoledgment
*/

int I2C_Receive_x(unsigned char, unsigned char, unsigned char*);



// --------------------------------------------------------------------------
/* Envoie une donnes sur le bus I2C
 * Arguments :
 *  - Le premier paramtre est l'adresse de l'esclave
 *  - Le second paramtre est la donne  envoyer
 * 
 * Paramtre renvoy : 
 *   0 : Tout s'est bien pass
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2 : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/

int I2C_Send(unsigned char Addr, unsigned char Data);


// --------------------------------------------------------------------------
/* Envoie des donnes sur le bus I2C
 * Arguments :
 *  - Le premier paramtre est l'adresse de l'esclave
 *  - Le second paramtre est le nombre de donnes  transmettre [0:255]
 *  - Le dernier paramtre est un tableau contenant les donnes  envoyer
 * 
 * Parametre renvoy : 
 * 	- 1 : La transmission s'est bien passe
 *  - 0 : pas d'acknoledgment
*/

int I2C_Send_x(unsigned char, unsigned char, const unsigned char*);


#endif
