// Fichier LCD.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              Code source de la librairie             *
 *					de l'afficheur LCD					*
 *                                                      *
 * .Date de cration : 28/12/07                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour l'afficheur LCD       *
 *                                                      *
 * .Grandes modifications :                             *
 *  __________________________________________________  *
 * |   DATE   |   Auteur   |          Detail          | *
 * |--------------------------------------------------| *
 * | 28/12/07 | P.Lucidarme| Cration et code des     | *
 * |          |			   | fonctions: 			  | *
 * |          |            | -LCD_Init			      | *
 * |          |            | -LCD_BackLight			  | *
 * |          |            | -LCD_Cursor		      | *
 * |          |            | -LCD_Clear			      | *
 * |          |            | -LCD_GotoXY		      | *
 * |          |            | -LCD_WriteCommand	      | *
 * |          |            | -LCD_Putchar		      | *
 * |          |            | -LCD_Busy			      | *
 * |          |            | -LCD_Scroll		      | * 
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |__________|____________|__________________________| *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/

#include <p18f452.h>		// Librairie du PIC utilis
#include "LCD.h"			// Header de la librairie
//#include <stdarg.h>			// Librairie pour les passages d'arguments variables (pour le printf)
#include <delays.h>

// Variables globales
unsigned char LCD_X,LCD_Y;	// Ces variables servent  mmoriser la position du curseur sur l'cran





// --------------------------------------------------------------------------
/* Fonction LCD_Init
 * Initialise l'cran LCD 
 * Curseur invisible, mode 8 bits ...
*/
void LCD_Init()
{
	// Commande des ports en E/S

 	TRIS_LCD_BL=0;			// Commande du rtro-clairage
	TRIS_LCD_RS=0;			// RS en sortie
	TRIS_LCD_RW=0;			// R/W en sortie
	TRIS_LCD_E=0;			// Bit Enable en sortie
	LCD_TRIS_DATA=0x00;		// Port de donnes en sortie

	// Ligne Enable au repos
	LCD_E=0;				// Place la ligne Enable au repos

	// Envoie des commandes de configuration sur l'afficheur
	LCD_WriteCommand(0x0C);		// Afficheur allum, Cursor teind, Pas de clignotement
	LCD_WriteCommand(0x38);		// Interface : 8 bits, 2 line display mode, caractres en 5x8 	
	LCD_Clear();				// Efface l'cran
}





// --------------------------------------------------------------------------
/* Fonction LCD_BackLight, commande le rtro-clairage
 * de l'afficheur LCD.
*/
void LCD_CURSOR (unsigned char State)
{
	LCD_WriteCommand (State);
}






// --------------------------------------------------------------------------
/* Fonction LCD_BackLight, commande le rtro-clairage
 * de l'afficheur LCD.
*/
void LCD_BackLight(unsigned char State)
{
	LCD_BL=State;
}






// --------------------------------------------------------------------------
/* Fonction LCD_Clear();
 * Efface l'cran de l'afficheur
*/
void LCD_Clear()
{
	LCD_WriteCommand (0x01);		// Envoie la commande 0x01 vers l'afficheur
	LCD_X=0;						// Le curseur est replac en 0,0
	LCD_Y=0;						// Il faut remettre  jour les variables
}



// --------------------------------------------------------------------------
/* Fonction LCD_GotoXY, place le curseur 
 * aux coordonnes X,Y (X tant le premier
 * paramtre, Y le second.
*/

void LCD_GotoXY (unsigned char X, unsigned char Y)
{	
	unsigned char YtoDDRAM[4]={0x00,0x40,0x14,0x54};		// Look Up Table pour la conversion X,y -> DDRAM adresse
	LCD_WriteCommand (0x80 | YtoDDRAM[Y] + X);				// Envoie la commande
	LCD_X=X;												// Met  jour les variables du curseur
	LCD_Y=Y;
}	





// --------------------------------------------------------------------------
/* Fonction LCD_PutChar : ecrit une donne dans l'afficheur
 * Le fonction gre automatiquement les retours chariots
 * et le scrolling de l'cran
 */
void LCD_PutChar (unsigned char Value)
{	
	if ((Value==10) || (Value==13))		// Retour  la ligne
	{
		LCD_Y++;					// On change de ligne
		if (LCD_Y==4)				// On est sur la dernire ligne
		{
			LCD_Scroll();			// On dcale les 3 dernires lignes vers le haut
			LCD_Y=3;				// On place le curseur sur la dernire	
		}
		LCD_X=0;					// On se place au dbut de la ligne (suivante)
		LCD_GotoXY(LCD_X,LCD_Y);	// On positionne le curseur  la nouvelle position
	}
	else
	{
		while (LCD_Busy());			// Attend que l'afficheur soit libre
		LCD_TRIS_DATA=0x00;			// Port de donnes en sortie		
		LCD_DATA=Value;				// Ecriture de la donne sur le port			
		LCD_RW=0;					// Ecriture		
		LCD_RS=1;					// Mode instruction		
//		Delay10TCY();				// Tempo pour respecter les temps de propagation ???
		LCD_E=1;					// Donne valide
		LCD_E=0;					// Ligne au repos	
		
		LCD_X++;					// On avance d'un caractre
		if (LCD_X==20)				// On est au bout de la ligne
		{
			LCD_Y++;				// On change de ligne
			if (LCD_Y==4)			// On est sur la dernire ligne
			{
				LCD_Scroll();		// On dcale les 3 dernires lignes vers le haut
				LCD_Y=3;			// On place le curseur sur la dernire	
			}
			LCD_X=0;				// On se place au dbut de la ligne (suivante)
			LCD_GotoXY(LCD_X,LCD_Y);		// On positionne le curseur  la nouvelle position
		}		
	}
	
}


// --------------------------------------------------------------------------
// Fonction LCD_WriteCommand : envoie la commande Value au processeur de l'cran
void LCD_WriteCommand (unsigned char Value)
{
	while (LCD_Busy());		// Attend que l'afficheur soit libre
	LCD_TRIS_DATA=0x00;		// Port de donnes en sortie
	LCD_DATA=Value;			// Ecriture de la donne sur le port	
	LCD_RS=0;				// Mode instruction
	LCD_RW=0;				// Ecriture	
	LCD_E=1;				// Donne valide
	LCD_E=0;				// Ligne au repo
}




// --------------------------------------------------------------------------
// Fonction LCD_Busy : renvoie vrai si l'cran est occup, faux sinon
unsigned char LCD_Busy()
{
	unsigned char Value;
	LCD_TRIS_DATA=0xFF;		// Port de donnes en entre	
	LCD_RW=1;				// Lecture
	LCD_RS=0;				// Mode instruction
	LCD_E=1;				// Donne valide	
  	Value=LCD_DATA;			// Ecriture de la donne sur le port	
	LCD_E=0;				// Ligne au repos
	return (Value&0x80);	// Retourne le bit Busy uniquement
}

// --------------------------------------------------------------------------
/* Fonction LCD_Scroll : dcalle les 3 dernires 
 * lignes d'une ligne vers le haut
 * Effectue un scrolling
 * ATTENTION : cette fonction ne modifie pas
 * les variables LCD_X et LCD_Y, elles doivent
 * tre mise  jour lors de l'appel
*/
void LCD_Scroll(void)
{
	char Caract;					// Caractre  copier
	unsigned char x,y;				// Position du curseur sur l'cran
	
	for (y=1;y<4;y++)				// Pour chaque ligne
	{		
		for (x=0;x<20;x++)			// Pour chaque colonne
		{
			// Lecture du caractre
			
			LCD_GotoXY(x,y);		// Place le curseur sur le caractre  copier
			while (LCD_Busy());		// Attend que l'afficheur soit libre
			LCD_TRIS_DATA=0xFF;		// Port de donnes en entre	
			LCD_RS=1;				// Mode instruction
			LCD_RW=1;				// Lecture
			LCD_E=1;				// Donne valide
			Caract=LCD_DATA;		// Ecriture de la donne sur le port	
			LCD_E=0;				// Ligne au repos
			
			// Recopie du caractre lu
			
			LCD_GotoXY(x,y-1);
			while (LCD_Busy());		// Attend que l'afficheur soit libre
			LCD_TRIS_DATA=0x00;		// Port de donnes en sortie
			LCD_DATA=Caract;		// Ecriture de la donne sur le port	
			LCD_RS=1;				// Mode instruction
			LCD_RW=0;				// Ecriture
//			Delay10TCY();			// Tempo pour respecter les temps de propagation ???
			LCD_E=1;				// Donne valide
			LCD_E=0;				// Ligne au repos	
			
		}	
	}
	
	// Efface la dernire ligne
	
	LCD_GotoXY(0,3);			// Place le curseur au dbut de la ligne
	for (x=0;x<20;x++)			// Affiche 20 espaces
	{		
		while (LCD_Busy());		// Attend que l'afficheur soit libre
		LCD_TRIS_DATA=0x00;		// Port de donnes en sortie
		LCD_RS=1;				// Mode instruction
		LCD_RW=0;				// Ecriture
		LCD_DATA=' ';			// Ecriture de la donne sur le port	
		LCD_E=1;				// Donne valide
		LCD_E=0;				// Ligne au repos				
	}	
}	