// Fichier LCD.h
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *               Hearder de la librairie                *
 *					de l'afficheur LCD					*
 *                                                      *
 * .Date de cration : 28/12/07                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - Prototypes des fonctions pour l'afficheur LCD *
 *                                                      *
 * .Grandes modifications :                             *
 *  __________________________________________________  *
 * |   DATE   |   Auteur   |          Detail          | *
 * |--------------------------------------------------| *
 * | 28/12/07 | P.Lucidarme| Cration et code des     | *
 * |          |			   | fonctions: 			  | *
 * |          |            | -LCD_Init			      | *
 * |          |            | -LCD_BackLight			  | *
 * |          |            | -LCD_Cursor		      | *
 * |          |            | -LCD_Clear			      | *
 * |          |            | -LCD_GotoXY		      | *
 * |          |            | -LCD_WriteCommand	      | *
 * |          |            | -LCD_Putchar		      | *
 * |          |            | -LCD_Busy			      | *
 * |          |            | -LCD_Scroll		      | * 
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |__________|____________|__________________________| *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/
 
 
#ifndef __LCD_H__
#define __LCD_H__

#define 	LCD_BL			PORTCbits.RC2	// Commande du rtro-clairage
#define		LCD_RS			PORTCbits.RC5	// Slection instructions / caractre
#define		LCD_RW			PORTCbits.RC1	// Selection lecture / ecriture
#define		LCD_E			PORTCbits.RC0	// Bit Enable (donne valide sur le port)
#define 	LCD_DATA		PORTD			// 8 bits de donnes


#define 	TRIS_LCD_BL		TRISCbits.TRISC2	// Commande du rtro-clairage
#define		TRIS_LCD_RS		TRISCbits.TRISC5	// Slection instructions / caractre
#define		TRIS_LCD_RW		TRISCbits.TRISC1	// Selection lecture / ecriture
#define		TRIS_LCD_E		TRISCbits.TRISC0	// Bit Enable (donne valide sur le port)
#define		LCD_TRIS_DATA	TRISD			// Sens des 8 bits de donne (lecture / criture)


// --------------------------------------------------------------------------
/* Fonction LCD_Init
 * COnfigure les ports en entres/sortie
 * Initialise l'cran LCD 
 * Curseur invisible, mode 8 bits ...
*/

void LCD_Init(void);



// --------------------------------------------------------------------------
/* Fonction LCD_BackLight, commande le rtro-clairage
 * de l'afficheur LCD.
 * L'tat peut prendre deux paramtres :
*/

#define 	LCD_BL_ON	0x01		// Allume le rtro-clairage
#define 	LCD_BL_OFF	0x00		// Eteind le rtro-clairage

void LCD_BackLight(unsigned char State);




// --------------------------------------------------------------------------
/* Fonction LCD_BackLight, commande le rtro-clairage
 * de l'afficheur LCD.
 * L'tat peut prendre un des trois paramtres suivant :
*/

#define 	LCD_CURSOR_ON		0x0E		// Curseur visible
#define 	LCD_CURSOR_OFF		0x0C		// Curseur invisible
#define 	LCD_CURSOR_BLINK	0x0F		// Curseur clignotant

void LCD_CURSOR (unsigned char);




// --------------------------------------------------------------------------
/* Fonction LCD_Clear();
 * Efface l'cran de l'afficheur
*/
void LCD_Clear(void);


// --------------------------------------------------------------------------
/* Fonction LCD_GotoXY, place le curseur 
 * aux coordonnes X,Y (X tant le premier
 * paramtre, Y le second.
*/
void LCD_GotoXY (unsigned char, unsigned char);



// --------------------------------------------------------------------------
/* Fonction LCD_PutChar : ecrit une donne dans l'afficheur
 * Le fonction gre automatiquement les retours chariots
 * et le scrolling de l'cran
 */
void LCD_PutChar (unsigned char Value);




// --------------------------------------------------------------------------
// Fonction LCD_WriteCommand : envoie la commande Value au processeur de l'cran
void LCD_WriteCommand (unsigned char Value);




// --------------------------------------------------------------------------
// Fonction LCD_Busy : renvoie vrai si l'cran est occup, faux sinon
unsigned char LCD_Busy(void);


// --------------------------------------------------------------------------
/* Fonction LCD_Scroll : dcalle les 3 dernires 
 * lignes d'une ligne vers le haut
 * Effectue un scrolling
*/
void LCD_Scroll(void);

#endif
