// Fichier MainBoard.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                Code source de la librairie           *
 *					de la carte principale 				*
 *                                                      *
 * .Date de cration : 29/12/07                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la carte principale   *
 *                                                      *
 * .Grandes modifications :                             *
 *  __________________________________________________  *
 * |   DATE   |   Auteur   |          Detail          | *
 * |--------------------------------------------------| *
 * | 28/12/07 | P.Lucidarme| Cration et code des     | *
 * |          |			   | fonctions: 			  | *
 * |          |            | - ConfigureProcesseur    | *
 * |          |            | - Joy_X                  | *
 * |          |            | - Joy_Y                  | * 
 * |          |            |                          | *  
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |__________|____________|__________________________| *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/

#include <p18f452.h>		// Librairie du PIC utilis
#include <adc.h>			// Librairie des convertisseurs analogiques numriques
#include <delays.h>			// Librairie des temporisation (pour les CAN)
#include <usart.h>			// Librairie pour la liaison srie

#include "lcd.h"			// Librairie de l'afficheur LCD
#include "MainBoard.h"		


#define		DELAY_SEL_ADC		5	// delays pour la slection du CAN ( x10 Tcy )



// --------------------------------------------------------------------------
/* Fonction ConfigureProcesseur
 * Configure les ports en entres/sorties
 * Configure les convertisseur analogique/numrique
 * ...
*/
void ConfigureProcesseur (void)
{
	
	// Configuration des ports en entre / sortie
	TRISA=0b111111;					// XXXXXX | XXXXXX | Joy    | XXXXXX | CANBat | CAN12V | Tel 30 | Tel150 
	TRISB=0b11110111;				// XXXXXX | XXXXXX | Joy_Sw | XXXXXX | RedLed | Sw_D   | Sw_G   | Jack
	TRISC=0b10000000; 				//    RX  |   TX   | XXXXXX |  SDA   |  SCL   | LCD_RS | LCD_RW | LCD_E |
 	TRISD=0b00000000;				// LCD_D7 | LCD_D6 | LCD_D5 | LCD_D4 | LCD_D3 | LCD_D2 | LCD_D1 | LCD_D0
	TRISE=0b011;					// LCD_BL | XXXXXX | XXXXXX


	// Configure le convertisseur analogique / numrique
	OpenADC(	ADC_FOSC_2 &		// Quartz divis par 2
				ADC_RIGHT_JUST &	// Ajustement  droite
				ADC_6ANA_0REF,		// 6 convertisseurs anamogiques, 2 ports numriques
				ADC_INT_OFF);		// Pas d'interruptions

				
	// Configure la liaison srie  115200 Bauds:
	OpenUSART	(	USART_TX_INT_OFF	&	// Pas d'interruption sur l'mission
					USART_RX_INT_OFF	&	// Pas d'interruption sur la rception
					USART_ASYNCH_MODE	&	// Mode asynchrone
					USART_EIGHT_BIT		& 	// Transmission sur 8 bits (1 octet)
					USART_CONT_RX		&	// Reception continue
					USART_BRGH_HIGH		,	// Haute vitesse
					10);					// (20e6 - 115200) / (16 * 115200)
	
	// Initialise l'afficheur LCD
	LCD_Init();						// Configure et initialise l'afficheur LCD

}	



// --------------------------------------------------------------------------
/* Fonction Joy_X
 * Retourne la position du Joystick en X
 * Valeur comprise entre 0 et 1023
 * ...
*/
int Joy_X()
{
	SetChanADC(ADC_CH4);		// Slectionne le canal 4
	Delay10TCYx(DELAY_SEL_ADC);	// Attends quelques cycles
	ConvertADC(); 				// Dmare la conversion
	while( BusyADC() ); 		// Attend la fin de la conversions
	return (0x3FF-ReadADC()); 	// Retourne le rsultat mis en form
}	




// --------------------------------------------------------------------------
/* Fonction Joy_Y
 * Retourne la position du Joystick en Y
 * Valeur comprise entre 0 et 1023
 * ...
*/
int Joy_Y()
{
	SetChanADC(ADC_CH5);		// Slectionne le canal 4
	Delay10TCYx(DELAY_SEL_ADC);	// Attends quelques cycles
	ConvertADC(); 				// Dmare la conversion
	while( BusyADC() ); 		// Attend la fin de la conversions
	return (ReadADC()); 		// Retourne le rsultat mis en form
}	





// --------------------------------------------------------------------------
/* Fonction Joy_Sw
 * Retourne l'tat du bouton du JoyStick
 * 	0= repos
 *	1= appuy
 * ...
*/
int Joy_Sw(void)
{
	return (!PORTBbits.RB5);
}	