// Fichier Robot.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                Code source de la librairie           *
 *					de la carte principale 				*
 *                                                      *
 * .Date de cration : 29/12/07                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la carte principale   *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/



#define 	ADR_I2C_ROULEAU_DROIT		0x4E		// Adresse IC du rouleau droit
#define 	ADR_I2C_ROULEAU_GAUCHE		0xBE		// Adresse IC du rouleau gauche

#define		CHECK_UP_VITESSE_ROULEAUX		50		// Vitesse applique sur les rouleaux
#define		CHECK_UP_MESURE_ROULEAUX		32		// Vitesse mesure minimum
#define 	CHECK_UP_DELAY					50		// Delai pour permettre au moteur d'atteindre sa vitesse 



// --------------------------------------------------------------------------
/* Commande les rouleaux
 * 	SpeedL : moteur gauche (vitesse entre -128 et +127)
 * 	SpeedR : moteur droit (vitesse entre -128 et +127)
 * Paramtre retourn : 
 *   0 : Tout s'est bien pass
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2 : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Commande_Rouleaux(char SpeedL, char SpeedR);	

// --------------------------------------------------------------------------
/* Commande le rouleau droit
 * le paramtre correspond  la vitesse  envoyer entre -128 et +127
 * Paramtre retourn : 
 *   0 : Tout s'est bien pass
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2 : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Commande_Rouleau_Droit(char Speed);	


// --------------------------------------------------------------------------
/* Mesure la vitesse du rouleau droit
 * La fonction mesure la vitesse du rouleau droit et retourne sa valeur
 *  entre -128 et +127 si tout s'est bien pass
 * 	0x0FFF en cas de problme dans la transmission
*/
int Mesure_Rouleau_Droit(void);


// --------------------------------------------------------------------------
/* Effectue un chek-up du rouleau droit
 * Lance le rouleau en avant et vrifie la vitesse   (erreurs 1 et 2)
 * Arrte le moteur et vrifie qu'il est  l'arrt   (erreurs 3 et 4)
 * Lance le rouleau en arrire et vrifie la vitesse (erreurs 5 et 6)
 * Pas de problme : on retourne 0 
*/
unsigned char Check_Up_Rouleau_Droit (void);


// --------------------------------------------------------------------------
/* Commande le rouleau gauche
 * le paramtre correspond  la vitesse  envoyer entre -128 et +127
 * Paramtre retourn : 
 *   0 : Tout s'est bien pass
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2 : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Commande_Rouleau_Gauche(char Speed);	


// --------------------------------------------------------------------------
/* Mesure la vitesse du rouleau gauche
 * La fonction mesure la vitesse du rouleau droit et retourne sa valeur
 *  entre -128 et +127 si tout s'est bien pass
 * 	0x0FFF en cas de problme dans la transmission
*/
int Mesure_Rouleau_Gauche(void);


// --------------------------------------------------------------------------
/* Effectue un chek-up du rouleau Gauche
 * Lance le rouleau en avant et vrifie la vitesse   (erreurs 1 et 2)
 * Arrte le moteur et vrifie qu'il est  l'arrt   (erreurs 3 et 4)
 * Lance le rouleau en arrire et vrifie la vitesse (erreurs 5 et 6)
 * Pas de problme : on retourne 0 
*/
unsigned char Check_Up_Rouleau_Gauche (void);
