// Fichier Serial.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                Code source de la librairie           *
 *					de la carte principale 				*
 *                                                      *
 * .Date de cration : 29/12/07                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la carte principale   *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/

#include <p18f452.h>
#include <usart.h>
#include <delays.h>


#include "serial.h"







// --------------------------------------------------------------------------
/* Initialise la carte propulsion (liaison srie)
 *  Pas de paramtre
*/
void Serial_Init()
{
	TRIS_RX=1;								// RX en entre (rception)
	TRIS_TX=0;								// TX en sortie (transmission)
	
	// Configure la liaison srie  57600 Bauds:
	OpenUSART	(	USART_TX_INT_OFF	&	// Pas d'interruption sur l'mission
					USART_RX_INT_OFF	&	// Interruption sur la rception
					USART_ASYNCH_MODE	&	// Mode asynchrone
					USART_EIGHT_BIT		& 	// Transmission sur 8 bits (1 octet)
					USART_CONT_RX		&	// Reception continue
					USART_BRGH_HIGH		,	// Haute vitesse
					21);					// 21 = (20e6 / (16 * 57600))-1 
	
	while (DataRdyUSART())					// Tant qu'il y a des caractres prsents
		ReadUSART();						// On les suprime du buffer
		
	Serial_ReStart();						// On vide la trame au cas ou !
}




// --------------------------------------------------------------------------
/* Envoie une srie de 0 et de 0xFF pour resynchroniser la carte propulsion
 *  Pas de paramtre
*/
void Serial_ReStart(void)
{
	char i;								// Indice pour parcourir le tableau
	unsigned char Trame[10];			// Trame  envoyer
	for (i=0;i<10;)						// Boucle pour crer la trame
	{
		Trame[i++]=0x00;				// Des 0 ...
		Trame[i++]=0xFF;				// ... suivis de 1		
	}
	Envoyer_Trame(Trame, 10);			// Envoi de la trame
}	





// --------------------------------------------------------------------------
/* Calcul le CRC de la trame
 *  Le premier paramtre est un pointeur vers le premier caractre de la trame
 *  Le second est le nombre de caractres
*/
unsigned char CRC (unsigned char* Trame,int NbCar)
{
	int i;
	unsigned char CRC=0;
	for (i=0;i<NbCar;i++)
		CRC^=Trame[i];
	return (CRC);		
}	




// --------------------------------------------------------------------------
/* Envoyer la trame sur la liaison srie
 *  Le premier est un pointeur vers le premier caractre de la trame
 *  Le second paramtre est le nombre de caractre  envoyer 
*/
void Envoyer_Trame(unsigned char* Trame, char NbCar)
{
	int i=0;
	while (BusyUSART());
	for (i=0;i<NbCar;i++)
	{
		WriteUSART (Trame[i]);
		while (BusyUSART());
		DELAY_USART		
	}	
}	





// --------------------------------------------------------------------------
/* Recevoir une trame sur la liaison srie
 *  Le premier est un pointeur vers le premier caractre de la trame
 *  Le second paramtre est le nombre de caractre  envoyer 
 *  Le dernier paramtre est la dure du TimeOut en ms
 * 
 * Paramtre renvoy : 
 *   0 : Tout s'est bien pass
 * 	-1 : On a pas reu tous les caractres avant la fin du TimeOut 
*/
char Recevoir_Trame(unsigned char* Trame, char NbCar,unsigned long int TimeOut)
{
	unsigned long int ms;					// Variable qui compte les millisecondes
	char i=0;								// Indice qui compte les caractres reus
	
	for (ms=0;ms<TimeOut*40;ms++)			// Une boucle pour les millisecondes
	{
		if (DataRdyUSART())					// On a reu un caractre
		{
			Trame[i++]=ReadUSART();			// On place la donne dans la trame			
			if (i==NbCar) return(0);		// Si on a reu tous les caractre attendu, on quitte
		}
	}
	return (-1);							// Fin du TimeOut, on envoi -1 pour signifier le problme
	
}	




// --------------------------------------------------------------------------
/* Vide le buffer de la liaison srie et supprime les erreurs ventuelles
 *  Aucun paramtre
*/
void Vide_Buffer_USART(void)
{
	
	if (RCSTAbits.OERR) 						// Si la liaison srie est en OVERRUN
	{
		RCSTAbits.CREN=0;						// Eteind le rcepteur ...
		RCSTAbits.CREN=1;						// ... pour le redmarrer		
	}
	while (DataRdyUSART())						// Tant qu'il y a des donnes ...
		ReadUSART();							// ... on les lit	
}



