// Fichier Telemetre.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                  librairie				            *
 *					de mesure de distance        		*
 *                                                      *
 * .Date de cration : 21/01/08                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la carte principale   *
 *                                                      *
 *														*
 *                                                      *
 *                                                      *
 * .Remarques :       									*
 *                                                      * 
 * ATTENTION : Les convertisseurs Analogiques/Numriques*
 * doivent avoir t configurs							*
 *                                                      *
 ********************************************************/


#include <adc.h>
#include <delays.h>

#include "Constante.h"

#include "Telemetres.h"
#include "CAN.h"
#include "I2C_Master.h"
#include "Propulsion.h"
#include "mystdio.h"

// --------------------------------------------------------------------------
/* Retourne le rsultat brut de la conversion du tlmtre Rouge
 *  0    : 0V
 *  1023 : 5V V
*/
int Telemetre_Rouge(void)
{
	return (CAN_Convert(ADC_CH1));
}



// --------------------------------------------------------------------------
/* Retourne le rsultat brut de la conversion du tlmtre Bleu
 *  0    : 0V
 *  1023 : 5V V
*/
int Telemetre_Bleu(void)
{
	return (CAN_Convert(ADC_CH0));
}




// --------------------------------------------------------------------------
/* Retourne la moyenne de N conversion du tlmtre Rouge
 *  0    : 0V
 *  1023 : 5V
*/
int Moyenne_Telemetre_Rouge(int N)
{
	unsigned long int Moyenne=0;						// Valeur moyenne
	int i;												// Indice pour la boucle
	for (i=0;i<N;i++)									// Effectue N mesures
	{
		Moyenne+=CAN_Convert(ADC_CH1);					// Conversion A/N
		Delay10TCYx(5);									// Petite tempo
	}
	return (Moyenne/N);									// On retourne la moyenne
}




// --------------------------------------------------------------------------
/* Retourne la moyenne de N conversion du tlmtre Bleu
 *  0    : 0V
 *  1023 : 5V
*/
int Moyenne_Telemetre_Bleu(int N)
{
	unsigned long int Moyenne=0;						// Valeur moyenne
	int i;												// Indice pour la boucle
	for (i=0;i<N;i++)									// Effectue N mesures
	{
		Moyenne+=CAN_Convert(ADC_CH0);					// Conversion A/N
		Delay10TCYx(5);									// Petite tempo
	}
	return (Moyenne/N);									// On retourne la moyenne
}





// --------------------------------------------------------------------------
/* Retourne la valeur du tlmtre avant gauche (mis en forme) 
 * Paramtre retourn : 
 *  >=0 : Tout s'est bien pass, c'est la valeur du tlmtre [0;255]
 * 	-1 	: Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2  : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Telemetre_Avant_Gauche (void)
{
	int ret,i=1;
	do 
	{
		Delay1KTCYx(i*5);
		ret=I2C_Receive(ADR_I2C_CARTE_SERVO);						// Retourne la valeur lue sur l'I2C
	}
	while (ret<0 && i++<10)	;
}	



// --------------------------------------------------------------------------
/* Retourne la valeur du tlmtre avant droit (mis en forme) 
 * Paramtre retourn : 
 *  >=0 : Tout s'est bien pass, c'est la valeur du tlmtre [0;255]
 * 	-1 	: Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2  : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Telemetre_Avant_Droit (void)
{
	unsigned char Data[2];										// Donnes brutes
	int Ret,i=1;				
	do															// Valeur de retour de l'envoie sur l'I2C
		Delay1KTCYx(i*5);
	while (I2C_Receive_x(ADR_I2C_CARTE_SERVO,2,Data) && i<10);	// Lecture des donnes
	if (i==10)
		return -1;												// En cas d'erreur sur l'I2C, on retourne -1		
	return Data[1];												// On retourne la deuxime donnes (c'est  dire le capteur avant droit)		
}		





// --------------------------------------------------------------------------
/* Retourne la valeur du tlmtre arrire gauche (mis en forme) 
 * Paramtre retourn : 
 *  >=0 : Tout s'est bien pass, c'est la valeur du tlmtre [0;255]
 * 	-1 	: Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2  : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Telemetre_Arriere_Gauche (void)
{
	unsigned char Data[3];										// Donnes brutes
		int Ret,i=1;				
	do															// Valeur de retour de l'envoie sur l'I2C
		Delay1KTCYx(i*5);
	while (I2C_Receive_x(ADR_I2C_CARTE_SERVO,3,Data) && i<10);	// Lecture des donnes
	if (i==10)
	{
		print_str ("Err Telemetre\n");
		return -1;												// En cas d'erreur sur l'I2C, on retourne -1		
	}
	return Data[2];	
}		


// --------------------------------------------------------------------------
/* Retourne la valeur du tlmtre arrire droit (mis en forme) le plus lev
 * Paramtre retourn : 
 *  >=0 : Tout s'est bien pass, c'est la valeur du tlmtre [0;255]
 * 	-1 	: Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2  : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Telemetre_Arriere (void)
{
	unsigned char Data[4];										// Donnes brutes
	int Ret,i=1;				
	do															// Valeur de retour de l'envoie sur l'I2C
		Delay1KTCYx(i*10);
	while (I2C_Receive_x(ADR_I2C_CARTE_SERVO,4,Data) && i<10);	// Lecture des donnes
	if (i==10)
	{
		print_str ("Err Telemetre\n");
		return -1;												// En cas d'erreur sur l'I2C, on retourne -1		
	}	
	if (Data[2]>Data[3])										// On retourne la plus grande donnes
		return Data[2];										
	else
		return Data[3];		
}	

// --------------------------------------------------------------------------
/* Retourne la valeur du tlmtre arrire droit (mis en forme) 
 * Paramtre retourn : 
 *  >=0 : Tout s'est bien pass, c'est la valeur du tlmtre [0;255]
 * 	-1 	: Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2  : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Telemetre_Arriere_Droit (void)
{
	unsigned char Data[4];										// Donnes brutes
	int Ret,i=1;				
	do															// Valeur de retour de l'envoie sur l'I2C
		Delay1KTCYx(i*5);
	while (I2C_Receive_x(ADR_I2C_CARTE_SERVO,4,Data) && i<10);	// Lecture des donnes
	if (i==10)
	{
		print_str ("Err Telemetre\n");
		return -1;												// En cas d'erreur sur l'I2C, on retourne -1		
	}
	return Data[3];												// On retourne la quatrime donnes (c'est  dire le capteur arrire droit)
}		



// --------------------------------------------------------------------------
/** tranche entre balle proche ou balle lointaine
  * Arguments :
  *		-> couleur (BLEUE ou ROUGE) : le tlmtre sur lequel il faut faire les mesures
  *		-> seuil_min : mesure au-dessous de laquelle on considre que la balle est loin
  *		-> seuil_max : mesure au-dessus de laquelle on considre que la balle est proche
  * Valeur renvoye :
  *		PROCHE (1) si les votes penchent en faveur d'une balle proche
  *		LOIN (-1)  si les votes penchent en faveur d'une balle lointaine
  */
int Vote_Telemetre(int couleur, int seuil_min, int seuil_max){
	int i;
	char plusMoitie=0;
	int MesureTelemetre;
	
	i=0;																// Nombre de mesure
	do																	
	{		
		if( couleur==BLEUE )
			MesureTelemetre=Moyenne_Telemetre_Bleu(1000);					// Effectue des mesures			
		else
			MesureTelemetre=Moyenne_Telemetre_Rouge(1000);					// Effectue des mesures			
		
		// est-on capable de dcider tout de suite ?
		if (MesureTelemetre>seuil_max) 
			return PROCHE;
		else if(MesureTelemetre<seuil_min) 
			return LOIN;
		
		// non, on fait voter :	
		else if( MesureTelemetre> (seuil_min+seuil_max)/2 )
				plusMoitie++;	
		
		i++;															// Incrmente le nombre de mesures
	}
	while (i<20);			// Jusqu' ce que l'on obtienne une valeur discriminante ou que l'on ai fait 20 mesures
	
	
	if( plusMoitie>i-plusMoitie )
		return PROCHE;
	else
		return LOIN;		
	
}



// --------------------------------------------------------------------------
/** tranche entre balle proche ou balle lointaine
  * Arguments :
  *		-> couleur (BLEUE ou ROUGE) : le tlmtre sur lequel il faut faire les mesures
  *		-> seuil_min : mesure au-dessous de laquelle on considre que la balle est loin
  *		-> seuil_max : mesure au-dessus de laquelle on considre que la balle est proche
  * Valeur renvoye :
  *		PROCHE (1) si les votes penchent en faveur d'une balle proche
  *		LOIN (-1)  si les votes penchent en faveur d'une balle lointaine
  
int OrienteVote_Telemetre(float angle, int couleur, int seuil_min, int seuil_max){
	int i,j;
	int plusMoitie=0;
	int MesureTelemetre;
	;
		
	
	
	Angle_Propulsion (angle+0.02,15);								// s'oriente un tout petit peu aprs l'angle demand
	for( j=0; j<3; j++ ){
		
		Angle_Propulsion (angle-0.02+j*0.01,15);								// s'oriente un tout petit peu avant l'angle demand
		while (IsMove_Propulsion());
		Stop_Propulsion();

		for( i=0; i<20; i++ ){		
			if( couleur==BLEUE )
				MesureTelemetre=Moyenne_Telemetre_Bleu(1000);					// Effectue des mesures			
			else
				MesureTelemetre=Moyenne_Telemetre_Rouge(1000);					// Effectue des mesures			
			
			// est-on capable de dcider tout de suite ?
			if (MesureTelemetre>seuil_max) 										// mesure>seuil_max -> c'est sr, la balle est proche
				return PROCHE;
			else if( MesureTelemetre<20 )										// mesure trop faible : ignorer
				continue;
			else if(MesureTelemetre<seuil_min) 									// mesure<seuil_min -> c'est sr, la balle est loin
				return LOIN;
			
			// non, on fait voter :	
			else if( MesureTelemetre> (seuil_min+seuil_max)/2 )
				plusMoitie++;	
			
			i++;	
			print_int(MesureTelemetre);															// Incrmente le nombre de mesures
			print_str(" ");
			//Delay1KTCYx(1);														// attendre un peu
		}
	
	}
	
	if( plusMoitie>60-plusMoitie )
		return PROCHE;
	else
		return LOIN;		
	
}
*/



// XM, 14/05
// --------------------------------------------------------------------------
/** recherche dans un secteur de +- deg un obstacle (une balle, le bord de la table, ou un distributeur ...) en vue des tlmtres
  * et oriente le robot le plus possible en face de cet obstacle 
  *	paramtres :
  * valeur renvoye :
  *		-> 1 si obstacle en face
  *		-> 0 si rien en face
  *		-> -1 si erreur I2C
  */
char OrienteMax_Telemetre( char deg ){
	float a_depart;
	float a=a_depart,amax=a_depart;
	int tele, telemax=0;
	int etat=0;
	
	a_depart =  A_Propulsion()/1000.0;
	Angle_Propulsion( a_depart+deg*PI/180.0, 15 );
	while( IsMove_Propulsion() );
		
	// un petit scan rapide :
	Angle_Propulsion( a_depart-deg*PI/180.0, 15 );
	
	do{	
		a = A_Propulsion()/1000.0;
		tele = Moyenne_Telemetre_Rouge(100) + Moyenne_Telemetre_Bleu(100);
		//print_strint( "tele= ", tele, 1 );
		if(	tele>telemax ){		
			telemax = tele;
			amax = a;
		}
	}while( IsMove_Propulsion() );
	

	// on revient dans l'autre sens
	Angle_Propulsion( a_depart+deg*PI/180.0, 15 );
	
	do{	
		tele = Moyenne_Telemetre_Rouge(50) + Moyenne_Telemetre_Bleu(50);
		switch( etat ){
			case 0:									// on n'est pas encore proche du niveau max
				if(	tele>=(3*telemax)/4 ){			// si on est proche du niveau max
					Angle_Propulsion( a_depart+deg*PI/180.0, 5 );	// ralentir
					etat=1;							// passer dans l'tat 1;
				}
				break;
				
			case 1:									// attente du niveau max
				if( tele>=telemax ){				// si on a atteint ou dpass le niveau prcdent
					telemax = tele;					// mmoriser le nouveau niveau max
					etat = 2;
				}
				break;
			
			case 2:
				if( tele>telemax ){					// si le niveau continue d'augmenter
					telemax = tele;				
				}else{								// si le niveau baisse
					Stop_Propulsion();				// arrter
				}
		} // fin switch
/*		
		print_int( etat );
		print_strint( "tele= ", tele, 0 );
		print_strint( "/", telemax, 1 );
*/
	}while( IsMove_Propulsion() );
	
	if( telemax<2*DISTANCE_OBSTACLE ){					// si on est loin d'un obstacle
		//Angle_Propulsion( a_depart, 20 );				// se repositionner  l'angle de dpart
		return 0;										//retourner 0
	}
	
	return 1;										//retourner 1
	
}

