// Fichier Telemetre.h
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                  librairie				            *
 *					de mesure de distance        		*
 *                                                      *
 * .Date de cration : 21/01/08                         *
 * .Auteur : Philippe Lucidarme                         *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la carte principale   *
 *                                                      *
 *														*
 *                                                      *
 *                                                      *
 * .Remarques :       									*
 *                                                      * 
 * ATTENTION : Les convertisseurs Analogiques/Numriques*
 * doivent avoir t configurs							*
 *                                                      *
 ********************************************************/

#define			ADR_I2C_CARTE_SERVO			0x52			// Adresse de la carte servo



// --------------------------------------------------------------------------
/* Retourne le rsultat brut de la conversion du tlmtre Rouge
 *  0    : 0V
 *  1023 : 5V V
*/
int Telemetre_Rouge(void);




// --------------------------------------------------------------------------
/* Retourne le rsultat brut de la conversion du tlmtre Bleu
 *  0    : 0V
 *  1023 : 5V V
*/
int Telemetre_Bleu(void);






// --------------------------------------------------------------------------
/* Retourne la moyenne de N conversion du tlmtre Rouge
 *  0    : 0V
 *  1023 : 5V
*/
int Moyenne_Telemetre_Rouge(int N);



// --------------------------------------------------------------------------
/* Retourne la moyenne de N conversion du tlmtre Bleu
 *  0    : 0V
 *  1023 : 5V
*/
int Moyenne_Telemetre_Bleu(int N);



// --------------------------------------------------------------------------
/* Retourne la valeur du tlmtre avant gauche (mis en forme) 
 * Paramtre retourn : 
 *  >=0 : Tout s'est bien pass, c'est la valeur du tlmtre [0;255]
 * 	-1 	: Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2  : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Telemetre_Avant_Gauche (void);


// --------------------------------------------------------------------------
/* Retourne la valeur du tlmtre avant droit (mis en forme) 
 * Paramtre retourn : 
 *  >=0 : Tout s'est bien pass, c'est la valeur du tlmtre [0;255]
 * 	-1 	: Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2  : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Telemetre_Avant_Droit (void);



// --------------------------------------------------------------------------
/* Retourne la valeur du tlmtre arrire gauche (mis en forme) 
 * Paramtre retourn : 
 *  >=0 : Tout s'est bien pass, c'est la valeur du tlmtre [0;255]
 * 	-1 	: Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2  : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Telemetre_Arriere_Gauche (void);


// --------------------------------------------------------------------------
/* Retourne la valeur du tlmtre arrire droit (mis en forme) le plus lev
 * Paramtre retourn : 
 *  >=0 : Tout s'est bien pass, c'est la valeur du tlmtre [0;255]
 * 	-1 	: Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2  : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Telemetre_Arriere (void);


// --------------------------------------------------------------------------
/* Retourne la valeur du tlmtre arrire droit (mis en forme) 
 * Paramtre retourn : 
 *  >=0 : Tout s'est bien pass, c'est la valeur du tlmtre [0;255]
 * 	-1 	: Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
 *  -2  : L'esclave n'a pas envoy d'acknoledgement aprs une donne
*/
int Telemetre_Arriere_Droit (void);




// --------------------------------------------------------------------------
/** tranche entre balle proche ou balle lointaine
  * Arguments :
  *		-> couleur (BLEUE ou ROUGE) : le tlmtre sur lequel il faut faire les mesures
  *		-> seuil_min : mesure au-dessous de laquelle on considre que la balle est loin
  *		-> seuil_max : mesure au-dessus de laquelle on considre que la balle est proche
  * Valeur renvoye :
  *		PROCHE (1) si les votes penchent en faveur d'une balle proche
  *		LOIN (-1)  si les votes penchent en faveur d'une balle lointaine
  */
int Vote_Telemetre(int couleur, int seuil_min, int seuil_max);


// --------------------------------------------------------------------------
/** recherche dans un secteur de +- deg un obstacle (une balle, le bord de la table, ou un distributeur ...) en vue des tlmtres
  * et oriente le robot le plus possible en face de cet obstacle 
  *	paramtres :
  * valeur renvoye :
  *		-> 1 si obstacle en face
  *		-> 0 si rien en face
  *		-> -1 si erreur I2C
  */
char OrienteMax_Telemetre( char deg );

// --------------------------------------------------------------------------
/** tranche entre balle proche ou balle lointaine
  * Arguments :
  *		-> couleur (BLEUE ou ROUGE) : le tlmtre sur lequel il faut faire les mesures
  *		-> seuil_min : mesure au-dessous de laquelle on considre que la balle est loin
  *		-> seuil_max : mesure au-dessus de laquelle on considre que la balle est proche
  * Valeur renvoye :
  *		PROCHE (1) si les votes penchent en faveur d'une balle proche
  *		LOIN (-1)  si les votes penchent en faveur d'une balle lointaine
  
int OrienteVote_Telemetre(float angle, int couleur, int seuil_min, int seuil_max);
*/
