// Fichier Strategie.H
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *                En-tte de la librairie           	*
 *					strategie							*
 *                                                      *
 * .Date de cration : 29/12/07                         *
 * .Auteur : Philippe Lucidarme                         *
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour la straegie   		*
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *                                                      *
 ********************************************************/
#ifndef __STRATEGIE_H__
#define __STRATEGIE_H__


#include "lcd.h"
#include "StartUp.h"
#include "JoyStick.h"
#include "Propulsion.h"
#include "rouleaux.h"
#include "IO.h"
#include "I2C_Master.h"
#include "barillet.h"
#include "Horloge.h"
#include "led.h"
#include "Telemetres.h"
#include "serial.h"
#include "canon.h"
#include "Tourelle.h"
#include "Ejecteur.h"
#include "Recal.h"
#include "Distributeur.h"



#define		DISTANCE_CONTACT			80				// Mesure du capteur au dela de laquelle on considre qu'il y a contact
#define		VIT_EVIT					50				// Vitesse lors de la manoeuvre d'vitement d'obstacle

#define		Y_DISTRI_BLANC		750				// Position en Y du barillet blanc


#define		VIT_LENTE					80				// Vitesse de dplacement lente
#define		VIT_RAPIDE					100				// Vitesse de dplacement rapide





#define		Y_CONTENEUR				1700				// Position en Y quand on va au conteneur

#define		X_INTER_DISTRIBUTEUR	2500				// Point de passage intermdiare quand on passe ct distributeur
#define		Y_INTER_DISTRIBUTEUR	800

#define		X_INTER_CONTENEUR		1100				// Point de passage intermdiare quand on passe ct conteneur
#define		Y_INTER_CONTENEUR		1600


#define 	DELTA_ANGLE				-0.3				// Dcalage vers le sens de la pente quand on dpose

														// Condition de changement de direction
#define		CONDITION_CHANGE_DIR	X_Propulsion()>1500 || Y_Propulsion()>800

#define		X_CONTENEUR_SECOURS		2700						// Position de docking au conteneur de secours



extern volatile unsigned char Nb_Depose_Adverse;


// --------------------------------------------------------------------------
/* Retourne du ct bleu align sur les deux distributeurs
 * Normalement on vite le robot adverse. Au bout de trois fois,
 * on passe en force
 * Arguments :
 *   Aucun
 *
 * Paramtre retourn :
 *   Aucun

unsigned char Goto_Cote_Rouge (void);
*/




// --------------------------------------------------------------------------
/* Retourne du ct bleu align sur les deux distributeurs en marche arrire
 * Normalement on vite le robot adverse. Au bout de trois fois,
 * on passe en force
 * Arguments :
 *   Aucun
 *
 * Paramtre retourn :
 *   Aucun

unsigned char Goto_Cote_Rouge_Arriere (void);
*/

// --------------------------------------------------------------------------
/* Va se placer contre le conteneur standard en marche arrire.
 * Normalement on vite le robot adverse. Au bout de trois fois,
 * on passe en force
 * Arguments :
 *  - La position en X du point de dpose
 *
 * Paramtre retourn :
 *   Aucun
*/
unsigned char Goto_Conteneur (int X);





// --------------------------------------------------------------------------
/* Avance et oriente lgrement le robot dans le sens de la pente
 * puis depose les balles
 * Arguments :
 * 	- la couleur que l'on va jouer durant ce match
 *
 * Paramtre retourn :
 *   0 : tout s'est bien pass,
 * 	-1 : Une collision s'est produite sur le BUS IC ou l'esclave n'a pas renvoy d'acknoledgment aprs l'adresse
*/
void Depose_Balles (char Couleur);




// --------------------------------------------------------------------------
/* Routine d'vitement du robot adverse :
 * On tourne le robot pour avoir l'adversaire soit sur un ct gauche ou droit  90
 * Puis on regarde s'il est devant ou derrire et on avance ou recule en consquence
 * La distance parcouru est dpendante de la nouvelle position du robot
 *
 * Arguments :
 * 	aucun
 *
 * Paramtre retourn :
 *  aucun
*/
void Evite_Robot_Adverse(unsigned char NbEvit);




//-----------------------------------------------------------------------
/* Le robot se place en dfense de notre ct en protgeant les balles
 *
 * Arguments :
 * 	aucun
 *
 * Paramtre retourn :
 *	 aucun
 */
void Defense_Bleu(void);



/** indique si l'adversaire est prsent devant
  * Arguments :
  *		aucun
  * Valeur renvoye :
  *		-1 si on ne sait pas o est la balise adverse
  *		0 si adversaire pas devant
  *		1 si adversaire loin devant
  *		2 si adversaire proche devant
  *		3 si adversaire au contact devant
  */
int Adversaire_Devant( void );
/** indique si l'adversaire est prsent derrire
  * Arguments :
  *		aucun
  * Valeur renvoye :
  *		-1 si on ne sait pas o est la balise adverse
  *		0 si adversaire pas derrire
  *		1 si adversaire loin derrire
  *		2 si adversaire proche derrire
  *		3 si adversaire au contact derrire
  */
int Adversaire_Derriere( void );

#endif
